.rc-table{font-size:12px;color:#666;transition:opacity .3s ease;position:relative;line-height:1.5;overflow:hidden}.rc-table-body-outer{margin:0!important}.rc-table-fixed-left tbody tr:last-child,.rc-table-fixed-right tbody tr:last-child{border-bottom:19px solid #f7f7f7!important}.rc-table-row.group{background:repeating-linear-gradient(-45deg,#fafafa,#fafafa 5px,#fff 0,#fff 10px)}.rc-table tr.group.rc-table-row-hover,.rc-table tr.rc-table-row.group:hover{background:repeating-linear-gradient(-45deg,#eaf8fe,#eaf8fe 5px,#fff 0,#fff 10px)}.rc-table-row.child{background:#fafafa}.rc-table-row.single{background:#fff}.rc-table-body::-webkit-scrollbar-track{-webkit-box-shadow:inset 0 0 6px rgba(0,0,0,.3)}.rc-table .rc-table-scroll{overflow:auto}.rc-table .rc-table-scroll table{width:auto;min-width:100%}.rc-table .rc-table-header{overflow:hidden;background:#f7f7f7;margin:0!important}.rc-table-fixed-header .rc-table-body{background:#fff;position:relative}.rc-table-fixed-header .rc-table-body-inner{height:100%;overflow:hidden!important}.rc-table-fixed-header .rc-table-scroll .rc-table-header{box-sizing:border-box}.rc-table-fixed-columns-in-body{visibility:hidden;pointer-events:none}.rc-table .rc-table-title{padding:0;border-top:1px solid #e9e9e9}.rc-table .rc-table-content{position:relative}.rc-table .rc-table-footer,.rc-table .rc-table-placeholder{padding:0;border-bottom:1px solid #e9e9e9}.rc-table .rc-table-placeholder{background:#fff;text-align:center;position:relative}.rc-table .rc-table-placeholder-fixed-columns{position:absolute;bottom:0;width:100%;background:transparent;pointer-events:none}.rc-table table{width:100%;border-collapse:collapse;text-align:left;border-spacing:0}.rc-table th{background:#f7f7f7;border:none;border-right:1px solid #e9e9e9;font-weight:700;transition:background .3s ease;overflow:hidden}.rc-table td{border:1px solid #e9e9e9;border-top:none;border-left:none;overflow:hidden}.rc-table td:empty:after{content:".";visibility:hidden}.rc-table tr{border:none;transition:all .3s ease}.rc-table tr.rc-table-row-hover,.rc-table tr:hover{background:#eaf8fe}.rc-table td,.rc-table th{padding:0;white-space:nowrap}.rc-table-expand-icon-col{width:34px}.rc-table-expanded-row-expand-icon,.rc-table-row-expand-icon{cursor:pointer;display:inline-block;width:16px;height:16px;text-align:center;line-height:16px;border:1px solid #e9e9e9;-webkit-user-select:none;-ms-user-select:none;user-select:none;background:#fff}.rc-table-expanded-row-spaced,.rc-table-row-spaced{visibility:hidden}.rc-table-expanded-row-spaced:after,.rc-table-row-spaced:after{content:"."}.rc-table-expanded-row-expanded:after,.rc-table-row-expanded:after{content:"-"}.rc-table-expanded-row-collapsed:after,.rc-table-row-collapsed:after{content:"+"}tr.rc-table-expanded-row,tr.rc-table-expanded-row:hover{background:#f7f7f7}.rc-table-column-hidden{display:none}.rc-table-next-columns-page,.rc-table-prev-columns-page{cursor:pointer;color:#666;z-index:1}.rc-table-next-columns-page:hover,.rc-table-prev-columns-page:hover{color:#2db7f5}.rc-table-next-columns-page-disabled,.rc-table-prev-columns-page-disabled{cursor:not-allowed;color:#999}.rc-table-next-columns-page-disabled:hover,.rc-table-prev-columns-page-disabled:hover{color:#999}.rc-table-prev-columns-page:before{content:"<"}.rc-table-next-columns-page{float:right}.rc-table-next-columns-page:before{content:">"}.rc-table-fixed-left,.rc-table-fixed-right{position:absolute;top:0;overflow:hidden}.rc-table-fixed-left table,.rc-table-fixed-right table{width:auto;background:#fff}.rc-table-fixed-left{left:0;box-shadow:4px 0 4px hsla(0,0%,39.2%,.1)}.rc-table-fixed-left .rc-table-body-inner{padding:0;margin:0}.rc-table-fixed-header .rc-table-fixed-left .rc-table-body-inner{padding-right:0}.rc-table-fixed-right{right:0;box-shadow:-4px 0 4px hsla(0,0%,39.2%,.1)}.rc-table-fixed-right .rc-table-expanded-row{color:transparent;pointer-events:none}.rc-table.rc-table-scroll-position-left .rc-table-fixed-left,.rc-table.rc-table-scroll-position-right .rc-table-fixed-right{box-shadow:none}
/*# sourceMappingURL=main.a2e245bd.chunk.css.map */